/******************************************
 *
 * $GAMGI/src/chem/gamgi_chem_mass.c
 *
 * Copyright (C) 2008 Carlos Pereira
 *
 * Distributed under the terms of the GNU
 * General Public License: $GAMGI/LICENSE
 *
 */

#include "gamgi_engine.h"
#include "gamgi_chem.h"
#include "gamgi_math.h"
#include "gamgi_global.h"

#define GAMGI_CHEM_MASS_YEAR   1000
#define GAMGI_CHEM_MASS_DAY    2000
#define GAMGI_CHEM_MASS_HOUR   3000
#define GAMGI_CHEM_MASS_MINUTE 4000
#define GAMGI_CHEM_MASS_SECOND 5000

/**********************************************************************
 * This list contains all naturally occurring isotopes, with their    *
 * relative abundances, plus all the isotopes with a half-life longer *
 * than one year (all elements until Cf except At, Rn, Fr), or, when  *
 * these do not exist, one day (Rn, plus Es, Fm, Md), one hour (At,   *
 * Lr, Rf, Db), one minute (Fr, No, Sg) or one second (Bh, Hs, Mt,    *
 * Ds, Rg). Some isotopes have both a natural abundance and a half    *
 * life decay, necessarily very long. Some of these isotopes          *
 * correspond to excited states (Rh, Ag, Sn, Ta, Re, Ir, Bi, Am).     *
 **********************************************************************/

const int gamgi_chem_mass_int[GAMGI_CHEM_ATOM_MAX + 1] = {

/***************************************
 * number of positions in double array *
 ***************************************/

GAMGI_CHEM_MASS_DOUBLE,

/****************************************************
 * starting offset for each element in double array *
 ****************************************************/

0, 8, 13, 18, 24, 29, 37, 42, 49, 52, 59, 65, 72, 78, 88, 91, 100, 108, 
121, 130, 152, 155, 169, 176, 187, 193, 210, 216, 235, 240, 251, 256, 
269, 272, 290, 295, 316, 323, 335, 338, 356, 368, 390, 400, 417, 426, 
444, 452, 479, 486, 513, 521, 546, 552, 573, 585, 604, 614, 629, 632, 
647, 657, 678, 689, 708, 717, 737, 743, 760, 766, 785, 798, 819, 829, 
850, 860, 884, 892, 912, 915, 935, 943, 962, 976, 983, 1000, 1005, 1033, 
1040, 1045, 1059, 1064, 1086, 1093, 1115, 1125, 1147, 1154, 1167, 1180, 
1187, 1194, 1204, 1208, 1212, 1216, 1220, 1233, 1240, 1244, 1251,
};

const double gamgi_chem_mass_double[GAMGI_CHEM_MASS_DOUBLE]= {

/*******
 * H 1 *
 *******/

1.00795,
1.007825, 99.9885,
2.014102, 0.0115,
3.016049, 12.32, GAMGI_CHEM_MASS_YEAR,

/********
 * He 2 *
 ********/

4.002602,
3.016029, 0.000137,
4.002603, 99.999863,

/********
 * 3 Li *
 ********/

6.941,
6.015123, 7.59,
7.016005, 92.41,
     
/********
 * 4 Be *
 ********/

9.012182,
9.012182, 100,
10.013534, 1.51E6, GAMGI_CHEM_MASS_YEAR,
        
/*******
 * 5 B *
 *******/

10.812,
10.012937, 19.9,
11.009305, 80.1,   
      
/*******
 * 6 C *
 *******/

12.0107,
12.000000, 98.93,
13.003355, 1.07,
14.003242, 5.73E3, GAMGI_CHEM_MASS_YEAR,
       
/*******
 * 7 N *
 *******/

14.0067,
14.003074, 99.632,
15.000109, 0.368,
                                                             
/*******
 * 8 O *
 *******/

15.9994,
15.994915, 99.757,
16.999132, 0.038,
17.999161, 0.205,

/*******
 * 9 F *
 *******/

18.998403,
18.998403, 100,
         
/*********
 * 10 Ne *
 *********/

20.1798,
19.992440, 90.48,
20.993847, 0.27,
21.991385, 9.25,

/*********
 * 11 Na *
 *********/

22.989769,
21.994436, 2.6027, GAMGI_CHEM_MASS_YEAR,
22.989769, 100,
        
/*********
 * 12 Mg *
 *********/

24.3051,
23.985042, 78.99,
24.985837, 10.00,
25.982593, 11.01,
      
/*********
 * 13 Al *
 *********/

26.981539,
25.986892, 7.17E5, GAMGI_CHEM_MASS_YEAR,
26.981539, 100,

/*********
 * 14 Si *
 *********/

28.0855,
27.976927, 92.2296,
28.976495, 4.6832,
29.973770, 3.0872,
31.974148, 170, GAMGI_CHEM_MASS_YEAR,

/********
 * 15 P *
 ********/

30.973762,
30.973762, 100,

/********
 * 16 S *
 ********/

32.065,
31.972071, 94.93,
32.971459, 0.76,
33.967867, 4.29,
35.967081, 0.02,

/*********
 * 17 Cl *
 *********/

35.453,
34.968853, 75.78,
35.968307, 3.01E5, GAMGI_CHEM_MASS_YEAR,
36.965903, 24.22,

/*********
 * 18 Ar *
 *********/

39.948,
35.967545, 0.3365,
37.962732, 0.0632,
38.964314, 269, GAMGI_CHEM_MASS_YEAR,
39.962383, 99.6003,
41.963047, 32.9, GAMGI_CHEM_MASS_YEAR,

/********
 * 19 K *
 ********/

39.0983,
38.963707, 93.2581,
39.963998, 0.0117, 1.277E9, GAMGI_CHEM_MASS_YEAR,
40.961826, 6.7302,

/*********
 * 20 Ca *
 *********/

40.078,
39.962591, 96.941, 5.9E21, GAMGI_CHEM_MASS_YEAR,
40.962278, 1.03E5, GAMGI_CHEM_MASS_YEAR,
41.958618, 0.647,
42.958767, 0.135,
43.955482, 2.086,
45.953693, 0.004, 2.5E15, GAMGI_CHEM_MASS_YEAR,
47.952534, 0.187, 4.3E19, GAMGI_CHEM_MASS_YEAR,

/*********
 * 21 Sc *
 *********/

44.955912,
44.955912, 100,

/*********
 * 22 Ti *
 *********/

47.867,
43.959690, 60.0, GAMGI_CHEM_MASS_YEAR,
45.952632, 8.25,
46.951763, 7.44,
47.947946, 73.72,
48.947870, 5.41,
49.944791, 5.18,

/********
 * 23 V *
 ********/

50.9415,
49.947159, 0.250, 1.5E17, GAMGI_CHEM_MASS_YEAR,
50.943960, 99.750,

/*********
 * 24 Cr *
 *********/

51.9961,
49.946044, 4.345, 1.8E17, GAMGI_CHEM_MASS_YEAR,
51.940508, 83.789,
52.940649, 9.501,
53.938880, 2.365,

/*********
 * 25 Mn *
 *********/

54.938046,
52.941290, 3.74E6, GAMGI_CHEM_MASS_YEAR,
54.938045, 100,

/*********
 * 26 Fe *
 *********/

55.845,
53.939611, 5.845, 3.1E22, GAMGI_CHEM_MASS_YEAR,
54.938293, 2.737, GAMGI_CHEM_MASS_YEAR,
55.934938, 91.754,
56.935394, 2.119,
57.933276, 0.282,
59.934072, 1.5E6, GAMGI_CHEM_MASS_YEAR,
      
/*********
 * 27 Co *
 *********/

58.933195,
58.933195, 100,
59.933817, 5.2714, GAMGI_CHEM_MASS_YEAR,
       
/*********
 * 28 Ni *
 *********/

58.6934,
57.935343, 68.0769, 7.00E20, GAMGI_CHEM_MASS_YEAR,
58.934347, 7.6E4, GAMGI_CHEM_MASS_YEAR,
59.930786, 26.2231,
60.931056, 1.1399,
61.928345, 3.6345,
62.929669, 100.1, GAMGI_CHEM_MASS_YEAR,
63.927966, 0.9256,
                                                  
/*********
 * 29 Cu *
 *********/

63.546,
62.929598, 69.17,
64.927790, 30.83,

/*********
 * 30 Zn *
 *********/

65.409,
63.929142, 48.63,
65.926033, 27.90,
66.927127, 4.10,
67.924844, 18.75,
69.925319, 0.62,
     
/*********
 * 31 Ga *
 *********/

69.723,
68.925574, 60.108,
70.924701, 39.892,
      
/*********
 * 32 Ge *
 *********/

72.64,
69.924247, 20.84,
71.922076, 27.54,  
72.923459, 7.73,
73.921178, 36.28,
75.921403, 7.61, 1.78E21, GAMGI_CHEM_MASS_YEAR,
     
/*********
 * 33 As *
 *********/

74.921597,
74.921597, 100,
       
/*********
 * 34 Se *
 *********/

78.96,
73.922476, 0.89,
75.919214, 9.37,
76.919914, 7.63,
77.917309, 23.77,
78.918499, 2.95E5, GAMGI_CHEM_MASS_YEAR,
79.916521, 49.61,
81.916699, 8.73, 1.08E20, GAMGI_CHEM_MASS_YEAR,
     
/*********
 * 35 Br *
 *********/

79.904,
78.918337, 50.69,
80.916291, 49.31,
      
/*********
 * 36 Kr *
 *********/

83.798,
77.920365, 0.35,   2.3E20, GAMGI_CHEM_MASS_YEAR,
79.916379, 2.28,
80.916592, 2.29E5, GAMGI_CHEM_MASS_YEAR,
81.913484, 11.58,
82.914136, 11.49,
83.911507, 57.00,
84.912527, 10.776, GAMGI_CHEM_MASS_YEAR,
85.910611, 17.30,
    
/*********
 * 37 Rb *
 *********/

85.4678,
84.911790, 72.17,
86.909181, 27.83, 4.88E10, GAMGI_CHEM_MASS_YEAR,

/*********
 * 38 Sr *
 *********/

87.62,
83.913425, 0.56,
85.909260, 9.86,
86.908877, 7.00,
87.905612, 82.58,
89.907738, 28.90, GAMGI_CHEM_MASS_YEAR,

/********
 * 39 Y *
 ********/

88.905848,
88.905848, 100,

/*********
 * 40 Zr *
 *********/

91.224,
89.904704, 51.45,
90.905646, 11.22,
91.905041, 17.15,
92.906476, 1.53E6, GAMGI_CHEM_MASS_YEAR,
93.906315, 17.38, 1.1E17, GAMGI_CHEM_MASS_YEAR,
95.908273, 2.80, 20E18, GAMGI_CHEM_MASS_YEAR,

/*********
 * 41 Nb *
 *********/

92.90638,
90.906996, 680, GAMGI_CHEM_MASS_YEAR,
91.907194, 3.47E7, GAMGI_CHEM_MASS_YEAR,
92.906378, 100,
93.907284, 2.03E4, GAMGI_CHEM_MASS_YEAR,

/*********
 * 42 Mo *
 *********/

95.94,
91.906811, 14.84, 1.90E20, GAMGI_CHEM_MASS_YEAR,
92.906813, 4.0E3, GAMGI_CHEM_MASS_YEAR,
93.905088, 9.25,
94.905842, 15.92,
95.904680, 16.68,
96.906022, 9.55,
97.905408, 24.13,
99.907478, 9.63, 7.8E18, GAMGI_CHEM_MASS_YEAR,

/*********
 * 43 Tc *
 *********/

97.907216,
96.906366, 4.21E6, GAMGI_CHEM_MASS_YEAR,
97.907216, 4.2E6, GAMGI_CHEM_MASS_YEAR,
98.906255, 2.111E5, GAMGI_CHEM_MASS_YEAR,

/*********
 * 44 Ru *
 *********/

101.07,
 95.907599, 5.54, 6.7E15, GAMGI_CHEM_MASS_YEAR,
 97.905288, 1.87,
 98.905939, 12.76,
 99.904220, 12.60,
100.905582, 17.06,
101.904349, 31.55,
103.905433, 18.62,

/*********
 * 45 Rh *
 *********/

102.905504,
100.906164, 3.3, GAMGI_CHEM_MASS_YEAR,
-101.906844, 3.742, GAMGI_CHEM_MASS_YEAR,
102.905504, 100,

/*********
 * 46 Pd *
 *********/

106.42,
101.905609, 1.02,
103.904036, 11.14,
104.905085, 22.33,
105.903486, 27.33,
106.905133, 6.5E6, GAMGI_CHEM_MASS_YEAR,
107.903892, 26.46,
109.905153, 11.72, 6.00E17, GAMGI_CHEM_MASS_YEAR,

/*********
 * 47 Ag *
 *********/

107.8682,
106.905098, 51.839,
-107.905957, 418, GAMGI_CHEM_MASS_YEAR,
108.904752, 48.161,

/*********
 * 48 Cd *
 *********/

112.412,
105.906460, 1.25, 4.10E20, GAMGI_CHEM_MASS_YEAR,
107.904185, 0.89, 4.10E17, GAMGI_CHEM_MASS_YEAR,
109.903002, 12.49,
110.904178, 12.80,
111.902758, 24.13,
112.904402, 12.22, 7.7E15, GAMGI_CHEM_MASS_YEAR,
113.903359, 28.73, 6.4E18, GAMGI_CHEM_MASS_YEAR,
115.904756, 7.49, 3.1E19, GAMGI_CHEM_MASS_YEAR,
       
/*********
 * 49 In *
 *********/

114.818,
112.904058, 4.29,
114.903879, 95.71, 4.41E14, GAMGI_CHEM_MASS_YEAR,
       
/*********
 * 50 Sn *
 *********/

118.710,
111.904819, 0.97,
113.902779, 0.66,
114.903342, 0.34,
115.901741, 14.54,
116.902952, 7.68,
117.901603, 24.22,
118.903308, 8.59,
119.902195, 32.58,
-120.904236, 44, GAMGI_CHEM_MASS_YEAR,
121.903439, 4.63,
123.905274, 5.79,
125.907653,  2.30E5, GAMGI_CHEM_MASS_YEAR,
      
/*********
 * 51 Sb *
 *********/

121.760,
120.903816, 57.21,
122.904214, 42.79,
124.905254, 2.75856, GAMGI_CHEM_MASS_YEAR,

      
/*********
 * 52 Te *
 *********/

127.60,
119.90402, 0.09, 2.2E16, GAMGI_CHEM_MASS_YEAR,
121.903044, 2.55,
122.904270, 0.89, 6.00E14, GAMGI_CHEM_MASS_YEAR,
123.902818, 4.74,
124.904431, 7.07,
125.903312, 18.84,
127.904463, 31.74,  2.2E24, GAMGI_CHEM_MASS_YEAR,
129.906224, 34.08, 7.90E20, GAMGI_CHEM_MASS_YEAR,

/********
 * 53 I *
 ********/

126.90447,
126.904473, 100,
128.904988, 1.57E7, GAMGI_CHEM_MASS_YEAR,
         
/*********
 * 54 Xe *
 *********/

131.294,
123.905893, 0.09, 4.8E16, GAMGI_CHEM_MASS_YEAR,
125.904274, 0.09,
127.903531, 1.92,
128.904780, 26.44,
129.903508, 4.08,
130.905082, 21.18,
131.904154, 26.89,
133.905395, 10.44,
135.907220, 8.87,
       
/*********
 * 55 Cs *
 *********/

132.905452,
132.905452, 100,
133.906718, 2.0652, GAMGI_CHEM_MASS_YEAR,
134.905977, 2.3E6, GAMGI_CHEM_MASS_YEAR,
136.907090, 30.1671, GAMGI_CHEM_MASS_YEAR,
         
/*********
 * 56 Ba *
 *********/

137.328,
129.906321, 0.106, 4.0E21, GAMGI_CHEM_MASS_YEAR,
131.905061, 0.101, 3.00E18, GAMGI_CHEM_MASS_YEAR,
133.904508, 2.417,
134.905689, 6.592,
135.904576, 7.854,
136.905827, 11.232,
137.905247, 71.698,

/*********
 * 57 La *
 *********/

138.90548,
136.906494, 6E4, GAMGI_CHEM_MASS_YEAR,
137.907112, 0.090, 1.02E11, GAMGI_CHEM_MASS_YEAR,
138.906353, 99.910,

/*********
 * 58 Ce *
 *********/

140.116,
135.907172, 0.185, 3.8E16, GAMGI_CHEM_MASS_YEAR,
137.905991, 0.251, 1.50E14, GAMGI_CHEM_MASS_YEAR,
139.905439, 88.450,
141.909244, 11.114, 5.0E16, GAMGI_CHEM_MASS_YEAR,

/*********
 * 59 Pr *
 *********/

140.907653,
140.907653, 100,

/*********
 * 60 Nd *
 *********/

144.242,
141.907723, 27.2,
142.909814, 12.2,
143.910087, 23.8,
144.912574, 8.3,
145.913117, 17.2,
147.916893, 5.7,
149.920891, 5.6,

/*********
 * 61 Pm *
 *********/

144.912749,
144.912749, 17.7, GAMGI_CHEM_MASS_YEAR,
145.914697, 5.53, GAMGI_CHEM_MASS_YEAR,
146.915139, 2.6234, GAMGI_CHEM_MASS_YEAR,

/*********
 * 62 Sm *
 *********/

150.36,
143.911999, 3.07,
145.913041, 1.03E8, GAMGI_CHEM_MASS_YEAR,
146.914898, 14.99,
147.914823, 11.24,
148.917185, 13.82,
149.917276, 7.38,
150.919932, 91, GAMGI_CHEM_MASS_YEAR,
151.919732, 26.75,
153.922209, 22.75,

/*********
 * 63 Eu *
 *********/

151.964,
149.919702, 36.9, GAMGI_CHEM_MASS_YEAR,
150.919850, 47.81,
151.921745, 13.537, GAMGI_CHEM_MASS_YEAR,
152.921230, 52.19,

/*********
 * 64 Gd *
 *********/

157.25,
151.919791, 0.20, 1.08E14, GAMGI_CHEM_MASS_YEAR,
153.920866, 2.18,
154.922622, 14.80,
155.922123, 20.47,
156.923960, 15.65,
157.924104, 24.84,
159.927054, 21.86, 3.1E+19, GAMGI_CHEM_MASS_YEAR,
 
/*********
 * 65 Tb *
 *********/

158.925347,
156.924025, 71, GAMGI_CHEM_MASS_YEAR,
157.925413, 180, GAMGI_CHEM_MASS_YEAR,
158.925347, 100,

/*********
 * 66 Dy *
 *********/

162.500,
153.924425, 3.0E6, GAMGI_CHEM_MASS_YEAR,
155.924284, 0.06, 1E18, GAMGI_CHEM_MASS_YEAR,
157.924409, 0.10,
159.925198, 2.34,
160.926933, 18.91,
161.926798, 25.51,
162.928731, 24.90,
163.929175, 28.18,

/*********
 * 67 Ho *
 *********/

164.930322,
162.928734, 4.570E3, GAMGI_CHEM_MASS_YEAR,
164.930322, 100,

/*********
 * 68 Er *
 *********/

167.259,
161.928778, 0.14, 1.40E14, GAMGI_CHEM_MASS_YEAR,
163.929200, 1.61,
165.930293, 33.61,
166.932048, 22.93,
167.932370, 26.78,
169.935464, 14.93,  3.20E17, GAMGI_CHEM_MASS_YEAR,

/*********
 * 69 Tm *
 *********/

168.934213,
168.934213, 100,
170.936429, 1.92, GAMGI_CHEM_MASS_YEAR,

/*********
 * 70 Yb *
 *********/

173.04,
167.933898, 0.13, 1.30E14, GAMGI_CHEM_MASS_YEAR,
169.934762, 3.04,
170.936326, 14.28,
171.936382, 21.83,
172.938211, 16.13,
173.938862, 31.83,
175.942572, 12.76, 1.60E17, GAMGI_CHEM_MASS_YEAR,

/*********
 * 71 Lu *
 *********/

174.967,
172.938931, 1.37, GAMGI_CHEM_MASS_YEAR,
173.940338, 3.31, GAMGI_CHEM_MASS_YEAR,
174.940772, 97.41,
175.942686, 2.59, 3.85E10, GAMGI_CHEM_MASS_YEAR,

/*********
 * 72 Hf *
 *********/

178.49,
171.939448, 1.87, GAMGI_CHEM_MASS_YEAR,
173.940046, 0.16, 2.0E15, GAMGI_CHEM_MASS_YEAR,
175.941409, 5.26,
176.943221, 18.60,
177.943699, 27.28,
178.945816, 13.62,
179.946550, 35.08,
181.950554, 8.90E6, GAMGI_CHEM_MASS_YEAR,

/*********
 * 73 Ta *
 *********/

180.94788,
178.945930, 1.82, GAMGI_CHEM_MASS_YEAR,
-179.947465, 0.012, 1.2E15, GAMGI_CHEM_MASS_YEAR,
180.947996, 99.988,
        
/********
 * 74 W *
 ********/

183.84,
179.946704, 0.12, 7.00E17, GAMGI_CHEM_MASS_YEAR,
181.948204, 26.50, 7.0E19, GAMGI_CHEM_MASS_YEAR,
182.950223, 14.31, 8.0E19, GAMGI_CHEM_MASS_YEAR,
183.950931, 30.64, 1.80E20, GAMGI_CHEM_MASS_YEAR,
185.954364, 28.43, 4.1E18, GAMGI_CHEM_MASS_YEAR,
      
/*********
 * 75 Re *
 *********/

186.207,
184.952955, 37.40,
-185.954986, 2.0E5, GAMGI_CHEM_MASS_YEAR,
186.955753, 62.60, 4.12E10, GAMGI_CHEM_MASS_YEAR,

/*********
 * 76 Os *
 *********/

190.23,
183.952489, 0.02, 5.6E13, GAMGI_CHEM_MASS_YEAR,
185.953838, 1.59, 2.0E15, GAMGI_CHEM_MASS_YEAR,
186.955751, 1.96,
187.955838, 13.24,
188.958148, 16.15,
189.958447, 26.26,
191.961481, 40.78, 9.8E12, GAMGI_CHEM_MASS_YEAR,
193.965182, 6.0, GAMGI_CHEM_MASS_YEAR,

/*********
 * 77 Ir *
 *********/

192.217,
190.960594, 37.3,
-191.962605, 241, GAMGI_CHEM_MASS_YEAR,
192.962926, 62.7,

/*********
 * 78 Pt *
 *********/

195.084,
189.959933, 0.014, 6.5E11, GAMGI_CHEM_MASS_YEAR,
191.961038, 0.782,
192.962987, 50, GAMGI_CHEM_MASS_YEAR,
193.962680, 32.967,
194.964791, 33.832,
195.964952, 25.242,
197.967893, 7.163, 3.20E14, GAMGI_CHEM_MASS_YEAR,

/*********
 * 79 Au *
 *********/

196.966569,
196.966569, 100,

/*********
 * 80 Hg *
 *********/

200.59,
193.965439, 444, GAMGI_CHEM_MASS_YEAR,
195.965833, 0.15, 2.5E18, GAMGI_CHEM_MASS_YEAR,
197.966769, 9.97,
198.968280, 16.87,
199.968326, 23.10,
200.970302, 13.18,
201.970643, 29.86,
203.973494, 6.87,

/*********
 * 81 Tl *
 *********/

204.3833,
202.972344, 29.524,
203.973864, 3.78, GAMGI_CHEM_MASS_YEAR,
204.974428, 70.476,

/*********
 * 82 Pb *
 *********/

207.2,
203.973044, 1.4, 1.40E17, GAMGI_CHEM_MASS_YEAR,
204.974482, 1.53E7, GAMGI_CHEM_MASS_YEAR,
205.974465, 24.1,
206.975897, 22.1,
207.976652, 52.4, 2E19, GAMGI_CHEM_MASS_YEAR,
209.984189, 22.20, GAMGI_CHEM_MASS_YEAR,

/*********
 * 83 Bi *
 *********/

208.980399,
206.978471, 32.9, GAMGI_CHEM_MASS_YEAR,
207.979742, 3.68E5, GAMGI_CHEM_MASS_YEAR,
208.980399, 100, 1.9E19, GAMGI_CHEM_MASS_YEAR,
-209.984120, 3.04E6, GAMGI_CHEM_MASS_YEAR,

/*********
 * 84 Po *
 *********/

208.982430,
207.981246, 2.898, GAMGI_CHEM_MASS_YEAR,
208.982430, 102, GAMGI_CHEM_MASS_YEAR,

/*********
 * 85 At *
 *********/

209.987149,
206.985784, 1.80, GAMGI_CHEM_MASS_HOUR,
207.986590, 1.63, GAMGI_CHEM_MASS_HOUR,
208.986174, 5.42, GAMGI_CHEM_MASS_HOUR,
209.987149, 100, 8.1, GAMGI_CHEM_MASS_HOUR,
210.987496, 7.215, GAMGI_CHEM_MASS_HOUR,

/*********
 * 86 Rn *
 *********/

222.017578,
222.017578, 100, 3.8235, GAMGI_CHEM_MASS_DAY,

/*********
 * 87 Fr *
 *********/

223.019736,
209.996408, 3.19, GAMGI_CHEM_MASS_MINUTE,
210.995537, 3.10, GAMGI_CHEM_MASS_MINUTE,
211.996202, 20.1, GAMGI_CHEM_MASS_MINUTE,
221.014256, 4.9, GAMGI_CHEM_MASS_MINUTE,
222.017552, 14.2, GAMGI_CHEM_MASS_MINUTE,
223.019736, 22.01, GAMGI_CHEM_MASS_MINUTE,
224.02326, 3.33, GAMGI_CHEM_MASS_MINUTE,
225.02557, 4.0, GAMGI_CHEM_MASS_MINUTE,
227.03184, 2.47, GAMGI_CHEM_MASS_MINUTE,

/*********
 * 88 Ra *
 *********/

226.025410,
226.025410, 1.601E3, GAMGI_CHEM_MASS_YEAR,
228.031070, 5.75, GAMGI_CHEM_MASS_YEAR,

/*********
 * 89 Ac *
 *********/

227.027752,
227.027752, 100, 21.772, GAMGI_CHEM_MASS_YEAR,

/*********
 * 90 Th *
 *********/

232.038055,
228.028741, 1.9116, GAMGI_CHEM_MASS_YEAR,
229.031762, 7.34E3, GAMGI_CHEM_MASS_YEAR,
230.033139, 7.538E4, GAMGI_CHEM_MASS_YEAR,
232.038055, 100, 1.406E10, GAMGI_CHEM_MASS_YEAR,

/*********
 * 91 Pa *
 *********/

231.035884,
231.035884, 100, 3.276E4, GAMGI_CHEM_MASS_YEAR,

/********
 * 92 U *
 ********/

238.02891,
232.037156, 68.9, GAMGI_CHEM_MASS_YEAR,
233.039635, 1.592E5, GAMGI_CHEM_MASS_YEAR,
234.040952, 0.0055, 2.456E5, GAMGI_CHEM_MASS_YEAR,
235.043930, 0.7200, 7.04E8, GAMGI_CHEM_MASS_YEAR,
236.045568, 2.342E7, GAMGI_CHEM_MASS_YEAR,
238.050788, 99.2745, 4.468E9, GAMGI_CHEM_MASS_YEAR,

/*********
 * 93 Np *
 *********/

237.048173,
236.04658, 1.55E5, GAMGI_CHEM_MASS_YEAR,
237.048173, 2.145E6, GAMGI_CHEM_MASS_YEAR,

/*********
 * 94 Pu *
 *********/

244.064204,
236.046058, 2.859, GAMGI_CHEM_MASS_YEAR,
238.049560, 87.7, GAMGI_CHEM_MASS_YEAR,
239.052163, 2.411E4, GAMGI_CHEM_MASS_YEAR,
240.053814, 6.562E3, GAMGI_CHEM_MASS_YEAR,
241.056852, 14.291, GAMGI_CHEM_MASS_YEAR,
242.058743, 3.75E+5, GAMGI_CHEM_MASS_YEAR,
244.064205, 8.01E+7, GAMGI_CHEM_MASS_YEAR,

/*********
 * 95 Am *
 *********/

243.061381,
241.056829, 432.3, GAMGI_CHEM_MASS_YEAR,
-242.059549, 141, GAMGI_CHEM_MASS_YEAR,
243.061381, 7.37E3, GAMGI_CHEM_MASS_YEAR,

/*********
 * 96 Cm *
 *********/

247.070355,
243.061389, 29.1, GAMGI_CHEM_MASS_YEAR,
244.062753, 18.10, GAMGI_CHEM_MASS_YEAR,
245.065491, 8.5E3, GAMGI_CHEM_MASS_YEAR,
246.067224, 4.76E3, GAMGI_CHEM_MASS_YEAR,
247.070355, 1.57E7, GAMGI_CHEM_MASS_YEAR,
248.072350, 3.49E5, GAMGI_CHEM_MASS_YEAR,
250.078357, 9.0E3, GAMGI_CHEM_MASS_YEAR,

/*********
 * 97 Bk *
 *********/

247.070308,
247.070308, 1380, GAMGI_CHEM_MASS_YEAR,
248.07310, 9, GAMGI_CHEM_MASS_YEAR,

/*********
 * 98 Cf *
 *********/

251.079588,
249.074854, 351, GAMGI_CHEM_MASS_YEAR,
250.076406, 13.09, GAMGI_CHEM_MASS_YEAR,
251.079588, 900, GAMGI_CHEM_MASS_YEAR,
252.081627, 2.646, GAMGI_CHEM_MASS_YEAR,

/*********
 * 99 Es *
 *********/

252.08299,
252.08299, 471.7, GAMGI_CHEM_MASS_DAY,
253.084825, 20.47, GAMGI_CHEM_MASS_DAY,
254.088023, 275.8, GAMGI_CHEM_MASS_DAY,
255.090273, 39.8, GAMGI_CHEM_MASS_DAY,

/**********
 * 100 Fm *
 **********/

257.095106,
253.085185, 3.00, GAMGI_CHEM_MASS_DAY,
257.095106, 100.5, GAMGI_CHEM_MASS_DAY,

/**********
 * 101 Md *
 **********/

258.098432,
258.098432, 51.5, GAMGI_CHEM_MASS_DAY,
260.10365, 31.9, GAMGI_CHEM_MASS_DAY,

/**********
 * 102 No *
 **********/

259.10103,
253.09068, 1.62, GAMGI_CHEM_MASS_MINUTE,
255.093241, 3.1, GAMGI_CHEM_MASS_MINUTE,
259.10103, 59, GAMGI_CHEM_MASS_MINUTE,

/**********
 * 103 Lr *
 **********/

262.10963,
262.10963, 4, GAMGI_CHEM_MASS_HOUR,

/**********
 * 104 Rf *
 **********/

265.11670,
265.11670, 13, GAMGI_CHEM_MASS_HOUR,

/**********
 * 105 Db *
 **********/

268.12546,
268.12546, 32, GAMGI_CHEM_MASS_HOUR,

/**********
 * 106 Sg *
 **********/

271.13348,
271.13348, 2.4, GAMGI_CHEM_MASS_MINUTE,

/**********
 * 107 Bh *
 **********/

267.12765,
264.1246, 1.4, GAMGI_CHEM_MASS_SECOND,
266.12694,  5, GAMGI_CHEM_MASS_SECOND,
267.12765, 22, GAMGI_CHEM_MASS_SECOND,
272.13804, 10, GAMGI_CHEM_MASS_SECOND,

/**********
 * 108 Hs *
 **********/

269.13406,
269.13406, 27, GAMGI_CHEM_MASS_SECOND,
270.13465, 3.6, GAMGI_CHEM_MASS_SECOND,

/**********
 * 109 Mt *
 **********/

276.15117,
276.15117, 0.72, GAMGI_CHEM_MASS_SECOND,

/**********
 * 110 Ds *
 **********/

280.15981,
280.15981, 12, GAMGI_CHEM_MASS_SECOND,
281.16207, 9.6, GAMGI_CHEM_MASS_SECOND,

/**********
 * 111 Rg *
 **********/

280.16448,
280.16448, 3.6, GAMGI_CHEM_MASS_SECOND,
};

static void static_row (GtkListStore *store,
char *label0, char *label1, char *label2, char *label3)
{
GtkTreeIter iter;

gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 
0, label0, 1, label1, 2, label2, 3, label3, -1);
}

void gamgi_chem_mass (GtkListStore *store, int element)
{
char label0[GAMGI_ENGINE_TOKEN];
char label1[GAMGI_ENGINE_TOKEN];
char label2[GAMGI_ENGINE_TOKEN];
char label3[GAMGI_ENGINE_TOKEN];
double value0, value1, value2, value3;
gamgi_bool excited;
int start, end;
int i;

if (element == FALSE) return;

/***************************************************
 * get first, last valid positions in double array *
 ***************************************************/

start = gamgi_chem_mass_int[element];
if (element == GAMGI_CHEM_ATOM_MAX) end = GAMGI_CHEM_MASS_DOUBLE - 1;
else end = gamgi_chem_mass_int[element + 1] - 1;

/*********************
 * show default mass *
 *********************/

sprintf (label0, "%.*f", gamgi->gamgi->mass, gamgi_chem_mass_double[start]);
strcpy (label1, "");
strcpy (label2, "");
strcpy (label3, "");
static_row (store, label0, label1, label2, label3);

i = start + 1;
do 
  {

  /*****************************************************************
   * show mass (label0), isotope number and excited state (label1) *
   *****************************************************************/

  excited = FALSE;
  value0 = gamgi_chem_mass_double[i];
  if (value0 < 0) { value0 = -value0; excited = TRUE; }

  sprintf (label0, "%.*f", gamgi->gamgi->mass, value0);
  sprintf (label1, "%d", (int) floor (value0 + 0.5));
  if (excited == TRUE) strcat (label1, "*");

  /**************************************************
   * show abundance (label2) and half-time (label3) *
   **************************************************/

  strcpy (label2, "");
  strcpy (label3, "");

  value1 = gamgi_chem_mass_double[i + 1];

  if (i + 2 > end)
    {
    sprintf (label2, "%.*f", GAMGI_MATH_DECIMAL_ABUNDANCE, value1);
    static_row (store, label0, label1, label2, label3);
    break;
    }

  value2 = gamgi_chem_mass_double[i + 2];

  if (value2 == GAMGI_CHEM_MASS_YEAR)
    {
    sprintf (label3, "%.*E y", GAMGI_MATH_DECIMAL_DECAY, value1);
    static_row (store, label0, label1, label2, label3);
    i += 3; continue;
    }
  if (value2 == GAMGI_CHEM_MASS_DAY)
    { 
    sprintf (label3, "%.*E d", GAMGI_MATH_DECIMAL_DECAY, value1);
    static_row (store, label0, label1, label2, label3);
    i += 3; continue;
    }
  if (value2 == GAMGI_CHEM_MASS_HOUR)
    { 
    sprintf (label3, "%.*E h", GAMGI_MATH_DECIMAL_DECAY, value1);
    static_row (store, label0, label1, label2, label3);
    i += 3; continue;
    }
  if (value2 == GAMGI_CHEM_MASS_MINUTE)
    {
    sprintf (label3, "%.*E m", GAMGI_MATH_DECIMAL_DECAY, value1);
    static_row (store, label0, label1, label2, label3);
    i += 3; continue;
    }
  if (value2 == GAMGI_CHEM_MASS_SECOND)
    {
    sprintf (label3, "%.*E s", GAMGI_MATH_DECIMAL_DECAY, value1);
    static_row (store, label0, label1, label2, label3);
    i += 3; continue;
    }

  /**********************************************
   * i + 3 is guaranteed to exist at this stage *
   **********************************************/

  value3 = gamgi_chem_mass_double[i + 3];

  if (value3 == GAMGI_CHEM_MASS_YEAR)
    {
    sprintf (label2, "%.*f", GAMGI_MATH_DECIMAL_ABUNDANCE, value1);
    sprintf (label3, "%.*E y", GAMGI_MATH_DECIMAL_DECAY, value2);
    static_row (store, label0, label1, label2, label3);
    i += 4; continue;
    }
    
  if (value3 == GAMGI_CHEM_MASS_DAY)
    {
    sprintf (label2, "%.*f", GAMGI_MATH_DECIMAL_ABUNDANCE, value1);
    sprintf (label3, "%.*E d", GAMGI_MATH_DECIMAL_DECAY, value2);
    static_row (store, label0, label1, label2, label3);
    i += 4; continue;
    }

  if (value3 == GAMGI_CHEM_MASS_HOUR)
    {
    sprintf (label2, "%.*f", GAMGI_MATH_DECIMAL_ABUNDANCE, value1);
    sprintf (label3, "%.*E h", GAMGI_MATH_DECIMAL_DECAY, value2);
    static_row (store, label0, label1, label2, label3);
    i += 4; continue;
    }
 
  if (value3 == GAMGI_CHEM_MASS_MINUTE)
    {
    sprintf (label2, "%.*f", GAMGI_MATH_DECIMAL_ABUNDANCE, value1);
    sprintf (label3, "%.*E m", GAMGI_MATH_DECIMAL_DECAY, value2);
    static_row (store, label0, label1, label2, label3);
    i += 4; continue;
    }

  if (value3 == GAMGI_CHEM_MASS_SECOND)
    {
    sprintf (label2, "%.*f", GAMGI_MATH_DECIMAL_ABUNDANCE, value1);
    sprintf (label3, "%.*E s", GAMGI_MATH_DECIMAL_DECAY, value2);
    static_row (store, label0, label1, label2, label3);
    i += 4; continue;
    }

  sprintf (label2, "%.*f", GAMGI_MATH_DECIMAL_ABUNDANCE, value1);
  static_row (store, label0, label1, label2, label3);
  i += 2;
  } while (i < end);

}
