/******************************************
 *
 * $GAMGI/src/chem/gamgi_chem_radius.c
 *
 * Copyright (C) 2008 Carlos Pereira
 *
 * Distributed under the terms of the GNU
 * General Public License: $GAMGI/LICENSE
 *
 */

#include "gamgi_engine.h"
#include "gamgi_chem.h"
#include "gamgi_global.h"

#define GAMGI_CHEM_IONIC_D  -10
#define GAMGI_CHEM_IONIC_LS -20
#define GAMGI_CHEM_IONIC_HS -30

#define GAMGI_CHEM_IONIC_COORDINATION 1000

/************************************************************************
 * All radius shown here came from http://www.webelements.com/,         *
 * where the original references can be consulted, except the           *
 * ionic radius, taken directly from Shannon's paper, Acta Cryst.       *
 * A32, 751 (1976):                                                     *
 *                                                                      *
 * 1) Half distance between atoms in its element natural state,         *
 * (most from L.E. Sutton (Ed.), Table of interatomic distances         *
 * and configuration in molecules and ions, Supplement 1956-1959,       *
 * Special publication No. 18, Chemical Society, London, UK,            *
 * 1965.). Available up to Cf (98), except Pm, At, Rn, Fr.              *
 *                                                                      *
 * 2) Effective atomic (from J.C. Slater, J. Chem. Phys. 1964, 39,      *
 * 3199), empirically derived by comparison of bond lengths in over     *
 * 1200 bond types in ionic, metallic, and covalent crystals and        *
 * molecules. Available up to Am (95) except He, Ne, Kr, Xe, At,        *
 * Rn, Fr.                                                              *
 *                                                                      *
 * 3) Calculated atomic (from E. Clementi, D.L.Raimondi, and W.P.       *
 * Reinhardt, J. Chem. Phys. 1963, 38, 2686), obtained from SCF         *
 * ab-initio calculations. Available up to Rn (86) except La, Ce.       *
 *                                                                      *
 * 4) Effective covalent (including from R.T. Sanderson in Chemical     *
 * Periodicity, Reinhold, New York, USA, 1962.), empirically obtained   *
 * by comparing distances between single-bonded equal atoms. Available  *
 * for all elements up to La (57), plus Lu (71) to Bi (83) plus Rn.     *
 *                                                                      *
 * 5) Calculated covalent, (from Beatriz Cordero et al, in "Covalent    *
 * radii revisited", Dalton Trans., 2008), arguably more consistent     *
 * than the effective covalent radius. Available up to Cm (96). For C,  *
 * there are radius available for sp3, sp2 and sp hybridization. For    *
 * Mn, Fe, Co there are radius available for low (LS) and high (HS)     *
 * spin configurations.                                                 *
 *                                                                      *
 * 6) Van der Waals (mainly from A. Bondi, J. Phys. Chem., 1964, 68,    *
 * 441.), established from contact distances between non-bonding atoms  *
 * in touching molecules or atoms.                                      *
 *                                                                      *
 * 7) Ionic effective (from R.D. Shannon, Acta Cryst. A32, 751, 1976.), *
 * empirically derived from about 1000 distances, taken mainly from     *
 * oxide and fluoride structures, plus a range of correlations. These   *
 * radius are a function of valence, coordination, mass (for H) and     *
 * low (LS) and high (HS) electronic spin (for Cr, Mn, Fe, Co, Ni).     *
 * Available for all elements up to Cf (98) expect He, Ne, Ar, Kr, Rn.  *
 *                                                                      *
 * To get the so-called ionic crystalline radius, suggested by Fumi     *
 * and Tosi and published also by Shannon, just sum 0.14 to the cation  *
 * and subtract 0.14 to the anion, so the cation-anion distance remain  *
 * unchanged. According to Shannon, it is felt that these crystal radii *
 * correspond more closely to the physical size of ions in a solid.     *
 * However, they might less efective in predicting the cation           *
 * coordination by using Pauling's first rule.                          *
 ************************************************************************/

const double gamgi_chem_radius_ionic_int[GAMGI_CHEM_RADIUS_IONIC_INT]= {

/************************************
 * number of positions in int array *
 ************************************/

GAMGI_CHEM_RADIUS_IONIC_INT,

/*************************************************
 * starting offset for each element in int array *
 *************************************************/

112, 118, 119, 124, 129, 134, 139, 147, 154, 162, 163, 172, 178, 183, 187,
194, 199, 207, 208, 217, 225, 229, 239, 252, 269, 296, 318, 334, 347, 359,
365, 370, 376, 382, 390, 400, 401, 411, 419, 425, 433, 444, 457, 465, 476,
483, 493, 508, 516, 521, 528, 531, 541, 550, 554, 562, 571, 579, 592, 600,
609, 614, 625, 637, 643, 652, 662, 668, 674, 682, 692, 697, 703, 712, 721,
731, 743, 750, 758, 766, 775, 784, 799, 806, 812, 815, 816, 819, 823, 826,
834, 844, 862, 876, 886, 897, 903, 909, 915, 916, 917, 918, 921, 922, 923,
924, 925, 926, 927, 928, 929,

/*******
 * H 1 *
 *******/

0,
+1,
1000,
2000,
2000, GAMGI_CHEM_IONIC_D,

/********
 * He 2 *
 ********/

-1,

/********
 * 3 Li *
 ********/

3,
+1,
4000,
6000,
8000,

/********
 * 4 Be *
 ********/

6,
+2,
3000,
4000,
6000,

/*******
 * 5 B *
 *******/

9,
+3,
3000,
4000,
6000,

/*******
 * 6 C *
 *******/

12,
+4,
3000,
4000,
6000,

/*******
 * 7 N *
 *******/

15,
-3,
4000,
+3,
6000,
+5,
3000,
6000,

/*******
 * 8 O *
 *******/

19,
-2,
2000,
3000,
4000,
6000,
8000,

/*******
 * 9 F *
 *******/
         
24,
-1,
2000,
3000,
4000,
6000,
+7,
6000,

/*********
 * 10 Ne *
 *********/

-1,

/*********
 * 11 Na *
 *********/

29,
+1,
4000,
5000,
6000,
7000,
8000,
9000,
12000,

/*********
 * 12 Mg *
 *********/

36,
+2,
4000,
5000,
6000,
8000,

/*********
 * 13 Al *
 *********/

40,
+3,
4000,
5000,
6000,

/*********
 * 14 Si *
 *********/

43,
+4,
4000,
6000,

/********
 * 15 P *
 ********/

45,
+3,
6000,
+5,
4000,
5000,
6000,

/********
 * 16 S *
 ********/

49,
+3,
4001,
5000,
6000,

/*********
 * 17 Cl *
 *********/

52,
-1,
6000,
+5,
3001,
+7,
4000,
6000,

/*********
 * 18 Ar *
 *********/

-1,

/********
 * 19 K *
 ********/

56,
+1,
4000, 
6000, 
7000, 
8000, 
9000, 
10000,
12000,

/*********
 * 20 Ca *
 *********/

63,
+2,
6000,
7000,
8000,
9000,
10000,
12000,

/*********
 * 21 Sc *
 *********/

69,
+3,
6000,
8000,

/*********
 * 22 Ti *
 *********/

71,
+2,
6000,
+3,
6000,
+4,
4000,
5000,
6000,
8000,

/********
 * 23 V *
 ********/

77,
+2,
6000,
+3,
6000,
+4,
5000,
6000,
8000,
+5,
4000,
5000,
6000,

/*********
 * 24 Cr *
 *********/

85,
+2,
6000, GAMGI_CHEM_IONIC_LS, GAMGI_CHEM_IONIC_HS,
+3,
6000,
+4,
4000,
6000,
+5,
4000,
6000,
8000,
+6,
4000,
6000,

/*********
 * 25 Mn *
 *********/

95,
+2,
4000, GAMGI_CHEM_IONIC_HS,
5000, GAMGI_CHEM_IONIC_HS,
6000, GAMGI_CHEM_IONIC_LS, GAMGI_CHEM_IONIC_HS,
7000, GAMGI_CHEM_IONIC_HS,
8000,
+3,
5000,
6000, GAMGI_CHEM_IONIC_LS, GAMGI_CHEM_IONIC_HS,
+4,
4000,
6000,
+5,
4000,
+6,
4000,
+7,
4000,
6000,

/*********
 * 26 Fe *
 *********/

110,
+2,
4000,
4002,
6000, GAMGI_CHEM_IONIC_LS, GAMGI_CHEM_IONIC_HS,
8000, GAMGI_CHEM_IONIC_HS,
+3,
4000, GAMGI_CHEM_IONIC_HS,
5000,
6000, GAMGI_CHEM_IONIC_LS, GAMGI_CHEM_IONIC_HS,
8000, GAMGI_CHEM_IONIC_HS,
+4,
6000,
+6,
4000,

/*********
 * 27 Co *
 *********/

122,
+2,
4000, GAMGI_CHEM_IONIC_HS,
5000,
6000, GAMGI_CHEM_IONIC_LS, GAMGI_CHEM_IONIC_HS,
8000,
+3,
6000, GAMGI_CHEM_IONIC_LS, GAMGI_CHEM_IONIC_HS,
+4,
4000,
6000,

/*********
 * 28 Ni *
 *********/

131,
+2,
4000,
4002,
5000,
6000,
+3,
6000, GAMGI_CHEM_IONIC_LS, GAMGI_CHEM_IONIC_HS,
+4,
6000, GAMGI_CHEM_IONIC_LS,

/*********
 * 29 Cu *
 *********/

138,
+1,
2000,
4000,
6000,
+2,
4000,
4002,
5000,
6000,
+3,
6000,

/*********
 * 30 Zn *
 *********/
     
146,
+2,
4000,
5000,
6000,
8000,

/*********
 * 31 Ga *
 *********/

150,
+3,
4000,
5000,
6000,

/*********
 * 32 Ge *
 *********/

153,
+2,
6000,
+4,
4000,
6000,

/*********
 * 33 As *
 *********/
 
156,
+3,
6000,
+5,
4000,
6000,

/*********
 * 34 Se *
 *********/
     
159,
-2,
6000,
+4,
6000,
+6,
4000,
6000,

/*********
 * 35 Br *
 *********/

163,
-1,
6000,
+3,
4002,
+5,
3001,
+7,
4000,
6000,

/*********
 * 36 Kr *
 *********/

-1,

/*********
 * 37 Rb *
 *********/

168,
+1,
6000,
7000,
8000,
9000,
10000,
11000,
12000,
14000,

/*********
 * 38 Sr *
 *********/

176,
+2,
6000,
7000,
8000,
9000,
10000,
12000,

/********
 * 39 Y *
 ********/

182,
+3,
6000,
7000,
8000,
9000,

/*********
 * 40 Zr *
 *********/

186,
+4,
4000,
5000,
6000,
7000,
8000,
9000,

/*********
 * 41 Nb *
 *********/

192,
+3,
6000,
+4,
6000,
8000,
+5,
4000,
6000,
7000,
8000,

/*********
 * 42 Mo *
 *********/

199,
+3,
6000,
+4,
6000,
+5,
4000,
6000,
+6,
4000,
5000,
6000,
7000,

/*********
 * 43 Tc *
 *********/

207,
+4,
6000,
+5,
6000,
+7,
4000,
6000,

/*********
 * 44 Ru *
 *********/

211,
+3,
6000,
+4,
6000,
+5,
6000,
+7,
4000,
+8,
4000,

/*********
 * 45 Rh *
 *********/

216,
+3,
6000,
+4,
6000,
+5,
6000,

/*********
 * 46 Pd *
 *********/

219,
+1,
2000,
+2,
4002,
6000,
+3,
6000,
+4,
6000,

/*********
 * 47 Ag *
 *********/

224,
+1,
2000,
4000,
4002,
5000,
6000,
7000,
8000,
+2,
4002,
6000,
+3,
4002,
6000,

/*********
 * 48 Cd *
 *********/

235,
+2,
4000,
5000,
6000,
7000,
8000,
12000,

/*********
 * 49 In *
 *********/

241,
+3,
4000,
6000,
8000,

/*********
 * 50 Sn *
 *********/

244,
+4,
4000,
5000,
6000,
7000,
8000,

/*********
 * 51 Sb *
 *********/
      
249,
+5,
6000,

/*********
 * 52 Te *
 *********/

250,
-2,
6000,
+4,
3000,
4000,
6000,
+6,
4000,
6000,

/********
 * 53 I *
 ********/

256,
-1,
6000,
+5,
3001,
6000,
+7,
4000,
6000,

/*********
 * 54 Xe *
 *********/

261,
+8,
4000,
6000,

/*********
 * 55 Cs *
 *********/
         
263,
+1,
6000,
8000,
9000,
10000,
11000,
12000,

/*********
 * 56 Ba *
 *********/

269,
+2,
6000,
7000,
8000,
9000,
10000,
11000,
12000,

/*********
 * 57 La *
 *********/

276,
+3,
6000,
7000,
8000,
9000,
10000,
12000,

/*********
 * 58 Ce *
 *********/

282,
+1,
6000,
7000,
8000,
9000,
10000,
12000,
+4,
6000,
8000,
10000,
12000,

/*********
 * 59 Pr *
 *********/

292,
+3,
6000,
8000,
9000,
+4,
6000,
8000,

/*********
 * 60 Nd *
 *********/

297,
+2,
8000,
9000,
+3,
6000,
8000,
9000,
12000,

/*********
 * 61 Pm *
 *********/

303,
+3,
6000,
8000,
9000,

/*********
 * 62 Sm *
 *********/

306,
+2,
7000,
8000,
9000,
+3,
6000,
7000,
8000,
9000,
12000,

/*********
 * 63 Eu *
 *********/

314,
+2,
6000,
7000,
8000,
9000,
10000,
+3,
6000,
7000,
8000,
9000,

/*********
 * 64 Gd *
 *********/
 
323,
+3,
6000,
7000,
8000,
9000,

/*********
 * 65 Tb *
 *********/

327,
+3,
6000,
7000,
8000,
9000,
+4,
6000,
8000,

/*********
 * 66 Dy *
 *********/

333,
+2,
6000,
7000,
8000,
+3,
6000,
7000,
8000,
9000,

/*********
 * 67 Ho *
 *********/

340,
+3,
6000,
8000,
9000,
10000,

/*********
 * 68 Er *
 *********/

344,
+3,
6000,
7000,
8000,
9000,

/*********
 * 69 Tm *
 *********/

348,
+2,
6000,
7000,
+3,
6000,
8000,
9000,

/*********
 * 70 Yb *
 *********/

353,
+2,
6000,
7000,
8000,
+3,
6000,
7000,
8000,
9000,

/*********
 * 71 Lu *
 *********/

360,
+3,
6000,
8000,
9000,

/*********
 * 72 Hf *
 *********/

363,
+4,
4000,
6000,
7000,
8000,

/*********
 * 73 Ta *
 *********/

367,
+3,
6000,
+4,
6000,
+5,
6000,
7000,
8000,

/********
 * 74 W *
 ********/
      
372,
+4,
6000,
+5,
6000,
+6,
4000,
5000,
6000,

/*********
 * 75 Re *
 *********/

377,
+4,
6000,
+5,
6000,
+6,
6000,
+7,
4000,
6000,

/*********
 * 76 Os *
 *********/

382,
+4,
6000,
+5,
6000,
+6,
5000,
6000,
+7,
6000,
+8,
4000,

/*********
 * 77 Ir *
 *********/

388,
+3,
6000,
+4,
6000,
+5,
6000,

/*********
 * 78 Pt *
 *********/

391,
+2,
4002,
6000,
+4,
6000,
+5,
6000,

/*********
 * 79 Au *
 *********/

395,
+1,
6000, 
+3,
4002,
6000,
+5,
6000,

/*********
 * 80 Hg *
 *********/

399,
+1,
3000,
6000,
+2,
2000,
4000,
6000,
8000,

/*********
 * 81 Tl *
 *********/

405,
+1,
6000,
8000,
12000,
+3,
4000,
6000,
8000,

/*********
 * 82 Pb *
 *********/

411,
+2,
4001,
6000,
7000,
8000,
9000,
10000,
11000,
12000,
+4,
4000,
5000,
6000,
8000,

/*********
 * 83 Bi *
 *********/

423,
+3,
5000,
6000,
8000,
+5,
6000,

/*********
 * 84 Po *
 *********/

427,
+4,
6000,
8000,
+6,
6000,

/*********
 * 85 At *
 *********/

430,
+7,
6000,

/*********
 * 86 Rn *
 *********/

-1,

/*********
 * 87 Fr *
 *********/

431,
+1,
6000,

/*********
 * 88 Ra *
 *********/

432,
+2,
8000,
12000,

/*********
 * 89 Ac *
 *********/

434,
+3,
6000,

/*********
 * 90 Th *
 *********/

435,
+4,
6000,
8000,
9000,
10000,
11000,
12000,

/*********
 * 91 Pa *
 *********/

441,
+3,
6000,
+4,
6000,
8000,
+5,
6000,
8000,
9000,

/********
 * 92 U *
 ********/

447,
+3,
6000,
+4,
6000,
7000,
8000,
9000,
12000,
+5,
6000,
7000,
+6,
2000,
4000,
6000,
7000,
8000,

/*********
 * 93 Np *
 *********/

460,
+2,
6000,
+3,
6000,
+4,
6000,
8000,
+5,
6000,
+6,
6000,
+7,
6000,

/*********
 * 94 Pu *
 *********/

467,
+3,
6000,
+4,
6000,
8000,
+5,
6000,
+6,
6000,

/*********
 * 95 Am *
 *********/

472,
+2,
7000,
8000,
9000,
+3,
6000,
8000,
+4,
6000,
8000,

/*********
 * 96 Cm *
 *********/

479,
+3,
6000,
+4,
6000,
8000,

/*********
 * 97 Bk *
 *********/

482,
+3,
6000,
+4,
6000,
8000,

/*********
 * 98 Cf *
 *********/

485,
+3,
6000,
+4,
6000,
8000,

/*********
 * 99 Es *
 *********/

-1,

/**********
 * 100 Fm *
 **********/

-1,

/**********
 * 101 Md *
 **********/

-1,

/**********
 * 102 No *
 **********/

488,
+2,
6000,

/**********
 * 103 Lr *
 **********/

-1,

/**********
 * 104 Rf *
 **********/

-1,

/**********
 * 105 Db *
 **********/

-1,

/**********
 * 106 Sg *
 **********/

-1,

/**********
 * 107 Bh *
 **********/

-1,

/**********
 * 108 Hs *
 **********/

-1,

/**********
 * 109 Mt *
 **********/

-1,

/**********
 * 110 Ds *
 **********/

-1,

/**********
 * 111 Rg *
 **********/

-1,

};

const double gamgi_chem_radius_ionic_double[GAMGI_CHEM_RADIUS_IONIC_DOUBLE]= {

/*******
 * H 1 *
 *******/

-0.38,
-0.18,
-0.10,

/********
 * He 2 *
 ********/

/********
 * 3 Li *
 ********/

0.590,
0.76,
0.92,
     
/********
 * 4 Be *
 ********/

0.16,
0.27,
0.45,
        
/*******
 * 5 B *
 *******/

0.01,
0.11,
0.27,
      
/*******
 * 6 C *
 *******/

-0.08,
0.15,
0.16,

/*******
 * 7 N *
 *******/

1.46,
0.16,
-0.104,
0.13,
                                                             
/*******
 * 8 O *
 *******/

1.35,
1.36,
1.38,
1.40,
1.42,

/*******
 * 9 F *
 *******/
         
1.285,
1.30,
1.31,
1.33,
0.08,

/*********
 * 10 Ne *
 *********/

/*********
 * 11 Na *
 *********/

0.99,
1.00,
1.02,
1.12,
1.18,
1.24,
1.39,
        
/*********
 * 12 Mg *
 *********/

0.57,
0.66,
0.720,
0.89,

/*********
 * 13 Al *
 *********/

0.39,
0.48,
0.535,

/*********
 * 14 Si *
 *********/

0.26,
0.400,

/********
 * 15 P *
 ********/

0.44,
0.17,
0.29,
0.38,

/********
 * 16 S *
 ********/

0.76,
0.80,
0.76,

/*********
 * 17 Cl *
 *********/

1.81,
0.12,
0.08,
0.27,

/*********
 * 18 Ar *
 *********/

/********
 * 19 K *
 ********/

1.37,
1.38,
1.46,
1.51,
1.55,
1.59,
1.64,

/*********
 * 20 Ca *
 *********/

1.00,
1.06,
1.12,
1.18,
1.23,
1.34,

/*********
 * 21 Sc *
 *********/

0.745,
0.870,

/*********
 * 22 Ti *
 *********/

0.86,
0.670,
0.42,
0.51,
0.605,
0.74,

/********
 * 23 V *
 ********/

0.79,
0.640,
0.53,
0.58,
0.72,
0.355,
0.46,
0.54,

/*********
 * 24 Cr *
 *********/

0.73, 
0.80,
0.615,
0.41,
0.55,
0.345,
0.49,
0.57,
0.26,
0.44,

/*********
 * 25 Mn *
 *********/

0.66,
0.75,
0.67,
0.830,
0.90,
0.96,
0.58,
0.58,
0.645,
0.39,
0.530,
0.33,
0.255,
0.25,
0.46,

/*********
 * 26 Fe *
 *********/

0.63,
0.64,
0.61,
0.780,
0.92,
0.49,
0.58,
0.55,
0.645,
0.78,
0.585,
0.25,

/*********
 * 27 Co *
 *********/

0.58,
0.67,
0.65,
0.745,
0.90,
0.545,
0.61,
0.40,
0.53,
       
/*********
 * 28 Ni *
 *********/

0.55,
0.49,
0.63,
0.690,
0.56,
0.60,
0.48,

/*********
 * 29 Cu *
 *********/

0.46,
0.60,
0.77,
0.57,
0.57,
0.65,
0.73,
0.54,

/*********
 * 30 Zn *
 *********/
     
0.60,
0.68,
0.740,
0.90,

/*********
 * 31 Ga *
 *********/

0.47,
0.55,
0.620,
      
/*********
 * 32 Ge *
 *********/

0.73,
0.390,
0.530,
     
/*********
 * 33 As *
 *********/
 
0.58,
0.335,
0.46,

/*********
 * 34 Se *
 *********/
     
1.98,
0.50,
0.28,
0.42,

/*********
 * 35 Br *
 *********/

1.96,
0.59,
0.31,
0.25,
0.39,
      
/*********
 * 36 Kr *
 *********/

/*********
 * 37 Rb *
 *********/

1.52,
1.56,
1.61,
1.63,
1.66,
1.69,
1.72,
1.83,

/*********
 * 38 Sr *
 *********/

1.18,
1.21,
1.26,
1.31,
1.36,
1.44,

/********
 * 39 Y *
 ********/

0.900,
0.96,
1.019,
1.075,

/*********
 * 40 Zr *
 *********/

0.59,
0.66,
0.72,
0.78,
0.84,
0.89,

/*********
 * 41 Nb *
 *********/

0.72,
0.68,
0.79,
0.48,
0.64,
0.69,
0.74,

/*********
 * 42 Mo *
 *********/

0.69,
0.650,
0.46,
0.61,
0.41,
0.50,
0.59,
0.73,

/*********
 * 43 Tc *
 *********/

0.645,
0.60,
0.37,
0.56,

/*********
 * 44 Ru *
 *********/

0.68,
0.620,
0.565,
0.38,
0.36,

/*********
 * 45 Rh *
 *********/

0.665,
0.60,
0.55,

/*********
 * 46 Pd *
 *********/

0.59,
0.64,
0.86,
0.76,
0.615,

/*********
 * 47 Ag *
 *********/

0.67,
1.00,
1.02,
1.09,
1.15,
1.22,
1.28,
0.79,
0.94,
0.67,
0.75,

/*********
 * 48 Cd *
 *********/

0.78,
0.87,
0.95,
1.03,
1.10,
1.31,
       
/*********
 * 49 In *
 *********/

0.62,
0.800,
0.92,
       
/*********
 * 50 Sn *
 *********/

0.55,
0.62,
0.690,
0.75,
0.81,

/*********
 * 51 Sb *
 *********/
      
0.60,

/*********
 * 52 Te *
 *********/

2.21,
0.52,
0.66,
0.97,
0.43,
0.56,

/********
 * 53 I *
 ********/

2.20,
0.44,
0.95,
0.42,
0.53,
         
/*********
 * 54 Xe *
 *********/

0.40,
0.48,

/*********
 * 55 Cs *
 *********/
         
1.67,
1.74,
1.78,
1.81,
1.85,
1.88,

/*********
 * 56 Ba *
 *********/

1.35,
1.38,
1.42,
1.47,
1.52,
1.57,
1.61,

/*********
 * 57 La *
 *********/

1.032,
1.10,
1.160,
1.216,
1.27,
1.36,

/*********
 * 58 Ce *
 *********/

1.01,
1.07,
1.143,
1.196,
1.25,
1.34,
0.87,
0.97,
1.07,
1.14,

/*********
 * 59 Pr *
 *********/

0.99,
1.126,
1.179,
0.85,
0.96,

/*********
 * 60 Nd *
 *********/

1.29,
1.35,
0.983,
1.109,
1.163,
1.27,

/*********
 * 61 Pm *
 *********/

0.97,
1.093,
1.144,

/*********
 * 62 Sm *
 *********/

1.22,
1.27,
1.32,
0.958,
1.02,
1.079,
1.132,
1.24,

/*********
 * 63 Eu *
 *********/

1.17,
1.20,
1.25,
1.30,
1.35,
0.947,
1.01,
1.066,
1.120,

/*********
 * 64 Gd *
 *********/
 
0.938,
1.00,
1.053,
1.107,

/*********
 * 65 Tb *
 *********/

0.923,
0.98,
1.040,
1.095,
0.76,
0.88,

/*********
 * 66 Dy *
 *********/

1.07,
1.13,
1.19,
0.912,
0.97,
1.027,
1.083,

/*********
 * 67 Ho *
 *********/

0.901,
1.015,
1.072,
1.12,

/*********
 * 68 Er *
 *********/

0.890,
0.945,
1.004,
1.062,

/*********
 * 69 Tm *
 *********/

1.03,
1.09,
0.880,
0.994,
1.052,

/*********
 * 70 Yb *
 *********/

1.02,
1.08,
1.14,
0.868,
0.925,
0.985,
1.042,

/*********
 * 71 Lu *
 *********/

0.861,
0.977,
1.032,

/*********
 * 72 Hf *
 *********/

0.58,
0.71,
0.76,
0.83,

/*********
 * 73 Ta *
 *********/

0.72,
0.68,
0.64,
0.69,
0.74,

/********
 * 74 W *
 ********/
      
0.66,
0.62,
0.42,
0.51,
0.60,

/*********
 * 75 Re *
 *********/

0.63,
0.58,
0.55,
0.38,
0.53,

/*********
 * 76 Os *
 *********/

0.630,
0.575,
0.49,
0.545,
0.525,
0.39,

/*********
 * 77 Ir *
 *********/

0.68,
0.625,
0.57,

/*********
 * 78 Pt *
 *********/

0.60,
0.80,
0.625,
0.57,

/*********
 * 79 Au *
 *********/

1.37,
0.68,
0.85,
0.57,

/*********
 * 80 Hg *
 *********/

0.97,
1.19,
0.69,
0.96,
1.02,
1.14,

/*********
 * 81 Tl *
 *********/

1.50,
1.59,
1.70,
0.75,
0.885,
0.98,

/*********
 * 82 Pb *
 *********/

0.98,
1.19,
1.23,
1.29,
1.35,
1.40,
1.45,
1.49,
0.65,
0.73,
0.775,
0.94,

/*********
 * 83 Bi *
 *********/

0.96,
1.03,
1.17,
0.76,

/*********
 * 84 Po *
 *********/

0.94,
1.08,
0.67,

/*********
 * 85 At *
 *********/

0.62,

/*********
 * 86 Rn *
 *********/

/*********
 * 87 Fr *
 *********/

1.80,

/*********
 * 88 Ra *
 *********/

1.48,
1.70,

/*********
 * 89 Ac *
 *********/

1.12,

/*********
 * 90 Th *
 *********/

0.94,
1.05,
1.09,
1.13,
1.18,
1.21,

/*********
 * 91 Pa *
 *********/

1.04,
0.90,
1.01,
0.78,
0.91,
0.95,

/********
 * 92 U *
 ********/

1.025,
0.89,
0.95,
1.00,
1.05,
1.17,
0.76,
0.84,
0.45,
0.52,
0.73,
0.81,
0.86,

/*********
 * 93 Np *
 *********/

1.10,
1.01,
0.87,
0.98,
0.75,
0.72,
0.71,

/*********
 * 94 Pu *
 *********/

1.00,
0.86,
0.96,
0.74,
0.71,

/*********
 * 95 Am *
 *********/

1.21,
1.26,
1.31,
0.975,
1.09,
0.85,
0.95,

/*********
 * 96 Cm *
 *********/

0.97,
0.85,
0.95,

/*********
 * 97 Bk *
 *********/

0.96,
0.83,
0.93,

/*********
 * 98 Cf *
 *********/

0.95,
0.821,
0.92,

/*********
 * 99 Es *
 *********/

/**********
 * 100 Fm *
 **********/

/**********
 * 101 Md *
 **********/

/**********
 * 102 No *
 **********/

1.1,

/**********
 * 103 Lr *
 **********/

/**********
 * 104 Rf *
 **********/

/**********
 * 105 Db *
 **********/

/**********
 * 106 Sg *
 **********/

/**********
 * 107 Bh *
 **********/

/**********
 * 108 Hs *
 **********/

/**********
 * 109 Mt *
 **********/

/**********
 * 110 Ds *
 **********/

/**********
 * 111 Rg *
 **********/

};

/***************************************
 * Distances in pm:                    *
 * 1) Bond length in Element-Element   *
 * 2) Atomic radius (empirical)        *
 * 3) Atomic radius (calculated)       *
 * 4) Covalent radius (empirical)      *
 * 5) Covalent radius (calculated)     *
 * 6) van der Waals radius             *
 *         Luis Veiros, Carlos Pereira *
 * Source: http://www.webelements.com/ *
 ***************************************/

const double gamgi_chem_radius_atomic_double[GAMGI_CHEM_RADIUS_ATOMIC_DOUBLE]= {

/*******
 * H 1 *
 *******/

74.130, 25, 53, 37, 31, 120,

/********
 * He 2 *
 ********/

300, 0, 31, 32, 28, 140,

/********
 * 3 Li *
 ********/

303.9, 145, 167, 134, 128, 182,

/********
 * 4 Be *
 ********/

222.6, 105, 112, 90, 96, 0,

/*******
 * 5 B *
 *******/

158.9, 85, 87, 82, 84, 0,

/*******
 * 6 C *
 *******/

142.6, 70, 67, 77, 76, 170,

/*******
 * 7 N *
 *******/

109.76, 65, 56, 75, 71, 55,

/*******
 * 8 O *
 *******/

120.741, 60, 48, 73, 66, 152,

/*******
 * 9 F *
 *******/

141.8, 50, 42, 71, 57, 47,

/*********
 * 10 Ne *
 *********/

313, 0, 38, 69, 58, 154,

/*********
 * 11 Na *
 *********/

371.6, 180, 190, 154, 166, 227,

/*********
 * 12 Mg *
 *********/

319.7, 150, 145, 130, 141, 173,

/*********
 * 13 Al *
 *********/

286.3, 125, 118, 118, 121, 0,

/*********
 * 14 Si *
 *********/

235.2, 110, 111, 111, 111, 210,

/********
 * 15 P *
 ********/

221, 100, 98, 106, 107, 180,

/********
 * 16 S *
 ********/

205, 100, 88, 102, 105, 180,

/*********
 * 17 Cl *
 *********/

189.1, 100, 79, 99, 102, 175,

/*********
 * 18 Ar *
 *********/

372, 0, 71, 97, 106, 188,

/********
 * 19 K *
 ********/

454.4, 220, 243, 196, 203, 275,

/*********
 * 20 Ca *
 *********/

394.7, 180, 194, 174, 176, 0,

/*********
 * 21 Sc *
 *********/

321.2, 160, 184, 144, 170, 0,

/*********
 * 22 Ti *
 *********/

289.6, 140, 176, 136, 160, 0,

/********
 * 23 V *
 ********/

262.2, 135, 171, 125, 153, 0,

/*********
 * 24 Cr *
 *********/

249.8, 140, 166, 127, 139, 0,

/*********
 * 25 Mn *
 *********/

273.1, 140, 161, 139, 139, 0,

/*********
 * 26 Fe *
 *********/

248.2, 140, 156, 125, 132, 0,

/*********
 * 27 Co *
 *********/

250.6, 135, 152, 126, 126, 0,

/*********
 * 28 Ni *
 *********/

249.2, 135, 149, 121, 124, 163,

/*********
 * 29 Cu *
 *********/

255.6, 135, 145, 138, 132, 140,

/*********
 * 30 Zn *
 *********/

266.5, 135, 142, 131, 122, 139,

/*********
 * 31 Ga *
 *********/

244.2, 130, 136, 126, 122, 187,

/*********
 * 32 Ge *
 *********/

245.0, 125, 125, 122, 120, 0,

/*********
 * 33 As *
 *********/

249, 115, 114, 119, 119, 185,

/*********
 * 34 Se *
 *********/

232.1, 115, 103, 116, 120, 190,

/*********
 * 35 Br *
 *********/

228.4, 115, 94, 114, 120, 185,

/*********
 * 36 Kr *
 *********/

404, 0, 88, 110, 116, 202,

/*********
 * 37 Rb *
 *********/

495, 235, 265, 211, 220, 0, 

/*********
 * 38 Sr *
 *********/

430.3, 200, 219, 192, 195, 0,

/********
 * 39 Y *
 ********/

355.1, 180, 212, 162, 190, 0,

/*********
 * 40 Zr *
 *********/

317.9, 155, 206, 148, 175, 0,

/*********
 * 41 Nb *
 *********/

285.8, 145, 198, 137, 164, 0,

/*********
 * 42 Mo *
 *********/

272.5, 145, 190, 145, 154, 0,

/*********
 * 43 Tc *
 *********/

270.3, 135, 183, 156, 147, 0,

/*********
 * 44 Ru *
 *********/

265.0, 130, 178, 126, 146, 0,

/*********
 * 45 Rh *
 *********/

269.0, 135, 173, 135, 142, 0,

/*********
 * 46 Pd *
 *********/

275.1, 140, 169, 131, 139, 163,

/*********
 * 47 Ag *
 *********/

288.9, 160, 165, 153, 145, 172,

/*********
 * 48 Cd *
 *********/

297.9, 155, 161, 148, 144, 158,

/*********
 * 49 In *
 *********/

325.1, 155, 156, 144, 142, 193,

/*********
 * 50 Sn *
 *********/

281.0, 145, 145, 141, 139, 217,

/*********
 * 51 Sb *
 *********/

290, 145, 133, 138, 139, 0,

/*********
 * 52 Te *
 *********/

286.4, 140, 123, 135, 138, 206,

/********
 * 53 I *
 ********/

266.6, 140, 115, 133, 139, 198,

/*********
 * 54 Xe *
 *********/

439, 0, 108, 130, 140, 216,

/*********
 * 55 Cs *
 *********/

530.9, 260, 298, 225, 244, 0,

/*********
 * 56 Ba *
 *********/

434.7, 215, 253, 198, 215, 0,

/*********
 * 57 La *
 *********/

373.9, 195, 0, 169, 207, 0,

/*********
 * 58 Ce *
 *********/

365.0, 185, 0, 0, 204, 0,

/*********
 * 59 Pr *
 *********/

364.0, 185, 247, 0, 203, 0,

/*********
 * 60 Nd *
 *********/

362.8, 185, 206, 0, 201, 0,

/*********
 * 61 Pm *
 *********/

0, 185, 205, 0, 199, 0,

/*********
 * 62 Sm *
 *********/

357.9, 185, 238, 0, 198, 0,

/*********
 * 63 Eu *
 *********/

398.9, 185, 231, 0, 198, 0,

/*********
 * 64 Gd *
 *********/

357.3, 180, 233, 0, 196, 0,

/*********
 * 65 Tb *
 *********/

352.5, 175, 225, 0, 194, 0,

/*********
 * 66 Dy *
 *********/

350.3, 175, 228, 0, 192, 0,

/*********
 * 67 Ho *
 *********/

348.6, 175, 226, 0, 192, 0,

/*********
 * 68 Er *
 *********/

346.8, 175, 226, 0, 189, 0,

/*********
 * 69 Tm *
 *********/

344.7, 175, 222, 0, 190, 0,

/*********
 * 70 Yb *
 *********/

388.0, 175, 222, 0, 187, 0,

/*********
 * 71 Lu *
 *********/

343.5, 175, 217, 160, 187, 0,

/*********
 * 72 Hf *
 *********/

312.7, 155, 208, 150, 175, 0,

/*********
 * 73 Ta *
 *********/

286, 145, 200, 138, 170, 0,

/********
 * 74 W *
 ********/

274.1, 135, 193, 146, 162, 0,

/*********
 * 75 Re *
 *********/

274.1, 135, 188, 159, 151, 0,

/*********
 * 76 Os *
 *********/

267.5, 130, 185, 128, 144, 0,

/*********
 * 77 Ir *
 *********/

271.4, 135, 180, 137, 141, 0,

/*********
 * 78 Pt *
 *********/

277.5, 135, 177, 128, 136, 175,

/*********
 * 79 Au *
 *********/

288.4, 135, 174, 144, 136, 166,

/*********
 * 80 Hg *
 *********/

300.5, 150, 171, 149, 132, 155,

/*********
 * 81 Tl *
 *********/

340.8, 190, 156, 148, 145, 196,

/*********
 * 82 Pb *
 *********/

350.0, 180, 154, 147, 146, 202,

/*********
 * 83 Bi *
 *********/

309, 160, 143, 146, 148, 0,

/*********
 * 84 Po *
 *********/

334.5, 190, 135, 0, 140, 0,

/*********
 * 85 At *
 *********/

0, 0, 127, 0, 150, 0,

/*********
 * 86 Rn *
 *********/

0, 0, 120, 145, 150, 0,

/*********
 * 87 Fr *
 *********/

0, 0, 0, 0, 260, 0,

/*********
 * 88 Ra *
 *********/

445.8, 215, 0, 0, 221, 0,

/*********
 * 89 Ac *
 *********/

375.6, 195, 0, 0, 215, 0,

/*********
 * 90 Th *
 *********/

359.5, 180, 0, 0, 206, 0,

/*********
 * 91 Pa *
 *********/

321.2, 180, 0, 0, 200, 0,

/********
 * 92 U *
 ********/

277, 175, 0, 0, 196, 186,

/*********
 * 93 Np *
 *********/

262, 175, 0, 0, 190, 0,

/*********
 * 94 Pu *
 *********/

302.6, 175, 0, 0, 187, 0,

/*********
 * 95 Am *
 *********/

345.0, 175, 0, 0, 180, 0,

/*********
 * 96 Cm *
 *********/

347.7, 0, 0, 0, 169, 0,

/*********
 * 97 Bk *
 *********/

339.8, 0, 0, 0, 0, 0,

/*********
 * 98 Cf *
 *********/

337.7, 0, 0, 0, 0, 0,

/*********
 * 99 Es *
 *********/

0, 0, 0, 0, 0, 0,

/**********
 * 100 Fm *
 **********/

0, 0, 0, 0, 0, 0,

/**********
 * 101 Md *
 **********/

0, 0, 0, 0, 0, 0,

/**********
 * 102 No *
 **********/

0, 0, 0, 0, 0, 0,

/**********
 * 103 Lr *
 **********/

0, 0, 0, 0, 0, 0,

/**********
 * 104 Rf *
 **********/

0, 0, 0, 0, 0, 0,

/**********
 * 105 Db *
 **********/

0, 0, 0, 0, 0, 0,

/**********
 * 106 Sg *
 **********/

0, 0, 0, 0, 0, 0,

/**********
 * 107 Bh *
 **********/

0, 0, 0, 0, 0, 0,

/**********
 * 108 Hs *
 **********/

0, 0, 0, 0, 0, 0,

/**********
 * 109 Mt *
 **********/

0, 0, 0, 0, 0, 0,

/**********
 * 110 Ds *
 **********/

0, 0, 0, 0, 0, 0,

/**********
 * 111 Rg *
 **********/

0, 0, 0, 0, 0, 0,

/************************************************************************
 * special cases (the first value is         6  C  76 sp3  73 sp2 69 sp *
 * stored in the normal covalent calculated  25 Mn 139  LS 161  HS      *
 * positions, the remaining are stored here  26 Fe 132  LS 152  HS      *
 * and handled as special cases:             27 Co 126  LS 150  HS      *
 ************************************************************************/

73, 69, 161, 152, 150,
};

static void static_row (GtkListStore *store,
char *label0, char *label1, char *label2, char *label3)
{
GtkTreeIter iter;

gtk_list_store_append (store, &iter);
gtk_list_store_set (store, &iter, 
0, label0, 1, label1, 2, label2, 3, label3, -1);
}

void gamgi_chem_radius_ionic (GtkListStore *store, int element)
{
char label0[GAMGI_ENGINE_TOKEN];
char label1[GAMGI_ENGINE_TOKEN];
char label2[GAMGI_ENGINE_TOKEN];
char label3[GAMGI_ENGINE_TOKEN];
int i, offset;
int end;
int value;
double radius;

if (element == FALSE) return;

/**********************************************
 * get start,end limits for int,double arrays *
 **********************************************/

i = gamgi_chem_radius_ionic_int[element];
offset = gamgi_chem_radius_ionic_int[i++];
if (offset == -1) return;

if (element == GAMGI_CHEM_ATOM_MAX) end = GAMGI_CHEM_RADIUS_IONIC_INT - 1;
else end = gamgi_chem_radius_ionic_int[element + 1] - 1;

/***************
 * get valence *
 ***************/

value = gamgi_chem_radius_ionic_int[i++];
while (i <= end)
  { 
  /**********************************
   * show valence, get coordination *
   **********************************/

  sprintf (label2, "%d", value);
  value = gamgi_chem_radius_ionic_int[i++];

  do
    {
    /*********************
     * show coordination *
     *********************/

    sprintf (label3, "%d", value/GAMGI_CHEM_IONIC_COORDINATION);
    if (value%GAMGI_CHEM_IONIC_COORDINATION == 1) strcat (label3, " Py");
    if (value%GAMGI_CHEM_IONIC_COORDINATION == 2) strcat (label3, " Sq");

    /*******************************************************
     * if next value contains HS for high spin, LS for low *
     * spin, or D for hydrogen, update and get next value  *
     *******************************************************/

    strcpy (label1, "Ionic Eff");
    value = gamgi_chem_radius_ionic_int[i++];
    if (value == GAMGI_CHEM_IONIC_HS)
      {
      strcat (label1, " HS");
      value = gamgi_chem_radius_ionic_int[i++];
      }
    else if (value == GAMGI_CHEM_IONIC_LS)
      {
      strcat (label1, " LS");
      value = gamgi_chem_radius_ionic_int[i++];
      }
    else if (value == GAMGI_CHEM_IONIC_D)
      {
      strcat (label1, " D");
      value = gamgi_chem_radius_ionic_int[i++];
      }

    /***************************
     * get radius and show row *
     ***************************/

    radius = gamgi_chem_radius_ionic_double[offset++];
    sprintf (label0, "%.*f", gamgi->gamgi->length, radius);
    static_row (store, label0, label1, label2, label3);

    /***********************************
     * LS HS sequence: get next value, *
     * get radius and show row         *
     *                                 *
     * HS LS sequence: does not exist  *
     ***********************************/

    if (value == GAMGI_CHEM_IONIC_HS)
      {
      strcpy (label1, "Ionic Eff HS");
      value = gamgi_chem_radius_ionic_int[i++];
      radius = gamgi_chem_radius_ionic_double[offset++];
      sprintf (label0, "%.*f", gamgi->gamgi->length, radius);
      static_row (store, label0, label1, label2, label3);
      }

    } while (value >= GAMGI_CHEM_IONIC_COORDINATION);
  }

}

void gamgi_chem_radius_atomic (GtkListStore *store, int element)
{
char label0[GAMGI_ENGINE_TOKEN];
char label1[GAMGI_ENGINE_TOKEN];
char label2[GAMGI_ENGINE_TOKEN];
char label3[GAMGI_ENGINE_TOKEN];
double value;

if (element == FALSE) return;

strcpy (label2, "");
strcpy (label3, "");

/***********************************
 * array starts at 0 for element 1 *
 ***********************************/

element--;

value = gamgi_chem_radius_atomic_double[6*element + 0];
if (value != 0)
  {
  sprintf (label0, "%.*f ", gamgi->gamgi->length, value/200);
  strcpy (label1, "Half-Distance");
  static_row (store, label0, label1, label2, label3);
  }

value = gamgi_chem_radius_atomic_double[6*element + 1];
if (value != 0)
  {
  sprintf (label0, "%.*f ", gamgi->gamgi->length, value/100);
  strcpy (label1, "Atomic Eff");
  static_row (store, label0, label1, label2, label3);
  }

value = gamgi_chem_radius_atomic_double[6*element + 2];
if (value != 0)
  {
  sprintf (label0, "%.*f ", gamgi->gamgi->length, value/100);
  strcpy (label1, "Atomic Cal");
  static_row (store, label0, label1, label2, label3);
  }

value = gamgi_chem_radius_atomic_double[6*element + 3];
if (value != 0)
  {
  sprintf (label0, "%.*f ", gamgi->gamgi->length, value/100);
  strcpy (label1, "Covalent Eff");
  static_row (store, label0, label1, label2, label3);
  }

/************************************************************************
 * special cases (the first value is         6  C  76 sp3  73 sp2 69 sp *
 * stored in the normal covalent calculated  25 Mn 139  LS 161  HS      *
 * positions, the remaining are stored here  26 Fe 132  LS 152  HS      *
 * and handled as special cases:             27 Co 126  LS 150  HS      *
 ************************************************************************/

value = gamgi_chem_radius_atomic_double[6*element + 4];
if (element == 5)
  {
  sprintf (label0, "%.*f ", gamgi->gamgi->length, value/100);
  strcpy (label1, "Covalent Cal sp3");
  static_row (store, label0, label1, label2, label3);

  value = gamgi_chem_radius_atomic_double[GAMGI_CHEM_RADIUS_ATOMIC_DOUBLE - 5];
  sprintf (label0, "%.*f ", gamgi->gamgi->length, value/100);
  strcpy (label1, "Covalent Cal sp2");
  static_row (store, label0, label1, label2, label3);

  value = gamgi_chem_radius_atomic_double[GAMGI_CHEM_RADIUS_ATOMIC_DOUBLE - 4];
  sprintf (label0, "%.*f ", gamgi->gamgi->length, value/100);
  strcpy (label1, "Covalent Cal sp");
  static_row (store, label0, label1, label2, label3);
  }
else if (element == 24)
  {
  sprintf (label0, "%.*f ", gamgi->gamgi->length, value/100);
  strcpy (label1, "Covalent Cal LS");
  static_row (store, label0, label1, label2, label3);

  value = gamgi_chem_radius_atomic_double[GAMGI_CHEM_RADIUS_ATOMIC_DOUBLE - 3];
  sprintf (label0, "%.*f ", gamgi->gamgi->length, value/100);
  strcpy (label1, "Covalent Cal HS");
  static_row (store, label0, label1, label2, label3);
  }
else if (element == 25)
  {
  sprintf (label0, "%.*f ", gamgi->gamgi->length, value/100);
  strcpy (label1, "Covalent Cal LS");
  static_row (store, label0, label1, label2, label3);
  
  value = gamgi_chem_radius_atomic_double[GAMGI_CHEM_RADIUS_ATOMIC_DOUBLE - 2];
  sprintf (label0, "%.*f ", gamgi->gamgi->length, value/100);
  strcpy (label1, "Covalent Cal HS");
  static_row (store, label0, label1, label2, label3);
  }
else if (element == 26)
  {
  sprintf (label0, "%.*f ", gamgi->gamgi->length, value/100);
  strcpy (label1, "Covalent Cal LS");
  static_row (store, label0, label1, label2, label3);
  
  value = gamgi_chem_radius_atomic_double[GAMGI_CHEM_RADIUS_ATOMIC_DOUBLE - 1];
  sprintf (label0, "%.*f ", gamgi->gamgi->length, value/100);
  strcpy (label1, "Covalent Cal HS");
  static_row (store, label0, label1, label2, label3);
  }
else if (value != 0)
  {
  sprintf (label0, "%.*f ", gamgi->gamgi->length, value/100);
  strcpy (label1, "Covalent Cal");
  static_row (store, label0, label1, label2, label3);
  }

value = gamgi_chem_radius_atomic_double[6*element + 5];
if (value != 0)
  {
  sprintf (label0, "%.*f ", gamgi->gamgi->length, value/100);
  strcpy (label1, "Van der Waals");
  static_row (store, label0, label1, label2, label3);
  }

}
